CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

CON

' At 80MHz the ADC sample resolutions and rates are as follows:
'
' sample   sample               
' bits       rate               
' ----------------              
' 9       156 KHz               
' 10       78 KHz               
' 11       39 KHz               
' 12     19.5 KHz               
' 13     9.77 KHz               
' 14     4.88 KHz               
                                
  bits = 12                     'try different values from table here
  attenuation = 2               'try 0-4

  averaging = 7                '2-power-n samples to compute average with
  
  sigma_delta_input     = 18 '20 ' 9
  sigma_delta_feedback  = 17 '19 ' 8


PUB SigmaDelta (sample)  : okay 
  stop
  okay := cap_cog := cognew(@asm_entry, sample)  + 1   'launch assembly program in a COG
    
PUB reset 
  audio_buf_ptr :=  audio_buf_base
    
PUB stop
  if cap_cog
    cogstop(cap_cog - 1)
    

DAT

'
'
' Assembly program
'
              org

asm_entry     mov       dira,asm_dira                 'make pin 8 (ADC) output

              movs      ctra,#sigma_delta_input       'POS W/FEEDBACK mode for CTRA
              movd      ctra,#sigma_delta_feedback
              movi      ctra,#%01001_000
              mov       frqa,#1

'              mov       xpos,#0
              mov      t1,par
              ADD      t1,#4
              rdlong   audio_buf_base,t1
              mov      video_buf_base,audio_buf_base
              ADD      video_buf_base,audio_words
              ADD      video_buf_base,audio_words
              mov      audio_buf_end,video_buf_base

              mov      audio_buf_ptr,audio_buf_base
                           
              mov       asm_cnt,cnt                     'prepare for WAITCNT loop
              add       asm_cnt,asm_cycles

              
:loop         waitcnt   asm_cnt,asm_cycles              'wait for next CNT value (timing is determinant after WAITCNT)

              mov       asm_sample,phsa                 'capture PHSA and get difference
              sub       asm_sample,asm_old
              add       asm_old,asm_sample


              add       average,asm_sample              'compute average periodically so that
              djnz      average_cnt,#:avgsame           'we can 0-justify samples
              mov       average_cnt,average_load
              shr       average,#averaging
              mov       asm_justify,average
              mov       average,#0                      'reset average for next averaging
:avgsame

              max       peak_min,asm_sample             'track min and max peaks for triggering
              min       peak_max,asm_sample
              djnz      peak_cnt,#:pksame
              mov       peak_cnt,peak_load
              mov       x,peak_max                      'compute min+12.5% and max-12.5%
              sub       x,peak_min
              shr       x,#3
              mov       trig_min,peak_min
              add       trig_min,x
              mov       trig_max,peak_max
              sub       trig_max,x
              mov       peak_min,bignum                 'reset peak detectors
              mov       peak_max,#0
:pksame
{
              cmp       mode,#0                 wz      'wait for negative trigger threshold
if_z          cmp       asm_sample,trig_min     wc
if_z_and_c    mov       mode,#1
if_z          jmp       #:loop

              cmp       mode,#1                 wz      'wait for positive trigger threshold
if_z          cmp       asm_sample,trig_max     wc
if_z_and_nc   mov       mode,#2
if_z          jmp       #:loop

              sub       asm_sample,asm_justify          'justify sample to bitmap center y
              sar       asm_sample,#attenuation         'this # controls attenuation (0=none)
              add       asm_sample,#384 / 2
              mins      asm_sample,#0
              maxs      asm_sample,#384 - 1
}
'              wrlong    asm_sample,par              'write sample back to Spin variable "sample" 
              wrword   asm_sample,audio_buf_ptr  'write sample back to Spin variable "sample" 
              add      audio_buf_ptr,#2         '(WRLONG introduces timing indeterminancy here..)
               
              CMP      audio_buf_ptr,audio_buf_end WC     
       IF_NC  mov      audio_buf_ptr,audio_buf_base
              

              jmp       #:loop                          'wait for next sample period

{              mov       x,xpos                          'xor old pixel off
              shl       x,#1
              add       x,asm_ypos
              rdword    y,x                             'get old pixel-y
              wrword    asm_sample,x                    'save new pixel-y
              mov       x,xpos
              call      #plot

              mov       x,xpos                          'xor new pixel on
              mov       y,asm_sample
              call      #plot

              add       xpos,#1                         'increment x position and mask
              and       xpos,#$1FF              wz
if_z          mov       mode,#0                         'if rollover, reset mode for trigger

              jmp       #:loop                          'wait for next sample period
'
'
' Plot
'
plot          mov       asm_mask,#1                     'compute pixel mask
              shl       asm_mask,x
              shl       y,#6                            'compute pixel address
              add       y,asm_pixels
              shr       x,#5
              shl       x,#2
              add       y,x
              rdlong    asm_data,y                      'xor pixel
              xor       asm_data,asm_mask
              wrlong    asm_data,y

plot_ret      ret                            
}
'
'
' Data
'
asm_cycles    long      5208 '|< bits - 1                     'sample time
asm_dira      long      |< sigma_delta_feedback
'asm_dira      long      $00000200                       'output mask
asm_pixels    long      0                               'pixel base (set at runtime)
asm_ypos      long      0                               'y positions (set at runtime)
average_cnt   long      1
peak_cnt      long      1
peak_load     long      512
mode          long      0
bignum        long      $FFFFFFFF
average_load  long      |< averaging
cap_cog       long      0
audio_words   long      512  'AudioFrameSize 
audio_buf_base      long       0
audio_buf_ptr       long       1

asm_justify   res       1
trig_min      res       1
trig_max      res       1
average       res       1
asm_cnt       res       1
asm_old       res       1
asm_sample    res       1
asm_mask      res       1
asm_data      res       1
'xpos          res       1
t1            res       1                        
video_buf_base      res       0
audio_buf_end       res       1
x             res       1
y             res       1
peak_min      res       1
peak_max      res       1